@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -34,16 +34,15 @@ import txaio
 txaio.use_twisted()
 
 import twisted.internet.protocol
-from twisted.internet.defer import maybeDeferred
 from twisted.internet.interfaces import ITransport
 from twisted.internet.error import ConnectionDone, ConnectionAborted, \
     ConnectionLost
 
+from autobahn.util import public
 from autobahn.wamp import websocket
-from autobahn.websocket.types import ConnectionRequest, ConnectionResponse, \
-    ConnectionDeny
+from autobahn.websocket.types import ConnectionRequest, ConnectionResponse, ConnectionDeny
 from autobahn.websocket import protocol
-from autobahn.twisted.util import peer2str
+from autobahn.twisted.util import peer2str, transport_channel_id
 
 from autobahn.websocket.compress import PerMessageDeflateOffer, \
     PerMessageDeflateOfferAccept, \
@@ -79,18 +78,18 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
     """
     Adapter class for Twisted WebSocket client and server protocols.
     """
-    peer = '<never connected>'
+
+    peer = u'<never connected>'
+
     log = txaio.make_logger()
 
     def connectionMade(self):
         # the peer we are connected to
         try:
-            peer = self.transport.getPeer()
+            self.peer = peer2str(self.transport.getPeer())
         except AttributeError:
             # ProcessProtocols lack getPeer()
-            self.peer = "process {}".format(self.transport.pid)
-        else:
-            self.peer = peer2str(peer)
+            self.peer = u'process:{}'.format(self.transport.pid)
 
         self._connectionMade()
         self.log.debug('Connection made to {peer}', peer=self.peer)
@@ -183,33 +182,33 @@ class WebSocketAdapterProtocol(twisted.internet.protocol.Protocol):
         self.transport.registerProducer(producer, streaming)
 
 
+@public
 class WebSocketServerProtocol(WebSocketAdapterProtocol, protocol.WebSocketServerProtocol):
     """
     Base class for Twisted-based WebSocket server protocols.
+
+    Implements :class:`autobahn.websocket.interfaces.IWebSocketChannel`.
     """
 
-    def _onConnect(self, request):
-        # onConnect() will return the selected subprotocol or None
-        # or a pair (protocol, headers) or raise an HttpException
-        res = maybeDeferred(self.onConnect, request)
+    log = txaio.make_logger()
 
-        res.addCallback(self.succeedHandshake)
-
-        def forwardError(failure):
-            if failure.check(ConnectionDeny):
-                return self.failHandshake(failure.value.reason, failure.value.code)
-            else:
-                self.log.debug("Unexpected exception in onConnect ['{failure.value}']", failure=failure)
-                return self.failHandshake("Internal server error: {}".format(failure.value), ConnectionDeny.INTERNAL_SERVER_ERROR)
-
-        res.addErrback(forwardError)
+    def get_channel_id(self, channel_id_type=u'tls-unique'):
+        """
+        Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
+        """
+        return transport_channel_id(self.transport, is_server=True, channel_id_type=channel_id_type)
 
 
+@public
 class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClientProtocol):
     """
     Base class for Twisted-based WebSocket client protocols.
+
+    Implements :class:`autobahn.websocket.interfaces.IWebSocketChannel`.
     """
 
+    log = txaio.make_logger()
+
     def _onConnect(self, response):
         self.onConnect(response)
 
@@ -217,6 +216,12 @@ class WebSocketClientProtocol(WebSocketAdapterProtocol, protocol.WebSocketClient
         self.log.debug("Starting TLS upgrade")
         self.transport.startTLS(self.factory.contextFactory)
 
+    def get_channel_id(self, channel_id_type=u'tls-unique'):
+        """
+        Implements :func:`autobahn.wamp.interfaces.ITransport.get_channel_id`
+        """
+        return transport_channel_id(self.transport, is_server=False, channel_id_type=channel_id_type)
+
 
 class WebSocketAdapterFactory(object):
     """
@@ -224,17 +229,22 @@ class WebSocketAdapterFactory(object):
     """
 
 
+@public
 class WebSocketServerFactory(WebSocketAdapterFactory, protocol.WebSocketServerFactory, twisted.internet.protocol.ServerFactory):
     """
     Base class for Twisted-based WebSocket server factories.
+
+    Implements :class:`autobahn.websocket.interfaces.IWebSocketServerChannelFactory`
     """
 
     def __init__(self, *args, **kwargs):
         """
-        In addition to all arguments to the constructor of
-        :class:`autobahn.websocket.protocol.WebSocketServerFactory`,
-        you can supply a `reactor` keyword argument to specify the
-        Twisted reactor to be used.
+
+        .. note::
+            In addition to all arguments to the constructor of
+            :meth:`autobahn.websocket.interfaces.IWebSocketServerChannelFactory`,
+            you can supply a ``reactor`` keyword argument to specify the
+            Twisted reactor to be used.
         """
         # lazy import to avoid reactor install upon module import
         reactor = kwargs.pop('reactor', None)
@@ -245,17 +255,22 @@ class WebSocketServerFactory(WebSocketAdapterFactory, protocol.WebSocketServerFa
         protocol.WebSocketServerFactory.__init__(self, *args, **kwargs)
 
 
+@public
 class WebSocketClientFactory(WebSocketAdapterFactory, protocol.WebSocketClientFactory, twisted.internet.protocol.ClientFactory):
     """
     Base class for Twisted-based WebSocket client factories.
+
+    Implements :class:`autobahn.websocket.interfaces.IWebSocketClientChannelFactory`
     """
 
     def __init__(self, *args, **kwargs):
         """
-        In addition to all arguments to the constructor of
-        :class:`autobahn.websocket.protocol.WebSocketClientFactory`,
-        you can supply a `reactor` keyword argument to specify the
-        Twisted reactor to be used.
+
+        .. note::
+            In addition to all arguments to the constructor of
+            :func:`autobahn.websocket.interfaces.IWebSocketClientChannelFactory`,
+            you can supply a ``reactor`` keyword argument to specify the
+            Twisted reactor to be used.
         """
         # lazy import to avoid reactor install upon module import
         reactor = kwargs.pop('reactor', None)
@@ -288,20 +303,19 @@ class WrappingWebSocketAdapter(object):
     """
 
     def onConnect(self, requestOrResponse):
-
         # Negotiate either the 'binary' or the 'base64' WebSocket subprotocol
         if isinstance(requestOrResponse, ConnectionRequest):
             request = requestOrResponse
             for p in request.protocols:
                 if p in self.factory._subprotocols:
-                    self._binaryMode = (p != 'base64')
+                    self._binaryMode = (p != u'base64')
                     return p
-            raise ConnectionDeny(ConnectionDeny.NOT_ACCEPTABLE, "this server only speaks %s WebSocket subprotocols" % self.factory._subprotocols)
+            raise ConnectionDeny(ConnectionDeny.NOT_ACCEPTABLE, u'this server only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
         elif isinstance(requestOrResponse, ConnectionResponse):
             response = requestOrResponse
             if response.protocol not in self.factory._subprotocols:
-                self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_PROTOCOL_ERROR, "this client only speaks %s WebSocket subprotocols" % self.factory._subprotocols)
-            self._binaryMode = (response.protocol != 'base64')
+                self._fail_connection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_PROTOCOL_ERROR, u'this client only speaks {0} WebSocket subprotocols'.format(self.factory._subprotocols))
+            self._binaryMode = (response.protocol != u'base64')
         else:
             # should not arrive here
             raise Exception("logic error")
@@ -311,14 +325,13 @@ class WrappingWebSocketAdapter(object):
 
     def onMessage(self, payload, isBinary):
         if isBinary != self._binaryMode:
-            self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_UNSUPPORTED_DATA, "message payload type does not match the negotiated subprotocol")
+            self._fail_connection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_UNSUPPORTED_DATA, u'message payload type does not match the negotiated subprotocol')
         else:
             if not isBinary:
                 try:
                     payload = b64decode(payload)
                 except Exception as e:
-                    self.failConnection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_INVALID_PAYLOAD, "message payload base64 decoding error: {0}".format(e))
-            # print("forwarding payload: {0}".format(binascii.hexlify(payload)))
+                    self._fail_connection(protocol.WebSocketProtocol.CLOSE_STATUS_CODE_INVALID_PAYLOAD, u'message payload base64 decoding error: {0}'.format(e))
             self._proto.dataReceived(payload)
 
     # noinspection PyUnusedLocal
@@ -326,7 +339,6 @@ class WrappingWebSocketAdapter(object):
         self._proto.connectionLost(None)
 
     def write(self, data):
-        # print("sending payload: {0}".format(binascii.hexlify(data)))
         # part of ITransport
         assert(type(data) == bytes)
         if self._binaryMode:
@@ -376,8 +388,7 @@ class WrappingWebSocketServerFactory(WebSocketServerFactory):
                  reactor=None,
                  enableCompression=True,
                  autoFragmentSize=0,
-                 subprotocol=None,
-                 debug=False):
+                 subprotocol=None):
         """
 
         :param factory: Stream-based factory to be wrapped.
@@ -386,15 +397,14 @@ class WrappingWebSocketServerFactory(WebSocketServerFactory):
         :type url: unicode
         """
         self._factory = factory
-        self._subprotocols = ['binary', 'base64']
+        self._subprotocols = [u'binary', u'base64']
         if subprotocol:
             self._subprotocols.append(subprotocol)
 
         WebSocketServerFactory.__init__(self,
                                         url=url,
                                         reactor=reactor,
-                                        protocols=self._subprotocols,
-                                        debug=debug)
+                                        protocols=self._subprotocols)
 
         # automatically fragment outgoing traffic into WebSocket frames
         # of this size
@@ -441,8 +451,7 @@ class WrappingWebSocketClientFactory(WebSocketClientFactory):
                  reactor=None,
                  enableCompression=True,
                  autoFragmentSize=0,
-                 subprotocol=None,
-                 debug=False):
+                 subprotocol=None):
         """
 
         :param factory: Stream-based factory to be wrapped.
@@ -451,15 +460,14 @@ class WrappingWebSocketClientFactory(WebSocketClientFactory):
         :type url: unicode
         """
         self._factory = factory
-        self._subprotocols = ['binary', 'base64']
+        self._subprotocols = [u'binary', u'base64']
         if subprotocol:
             self._subprotocols.append(subprotocol)
 
         WebSocketClientFactory.__init__(self,
                                         url=url,
                                         reactor=reactor,
-                                        protocols=self._subprotocols,
-                                        debug=debug)
+                                        protocols=self._subprotocols)
 
         # automatically fragment outgoing traffic into WebSocket frames
         # of this size
@@ -490,6 +498,7 @@ class WrappingWebSocketClientFactory(WebSocketClientFactory):
         return proto
 
 
+@public
 def connectWS(factory, contextFactory=None, timeout=30, bindAddress=None):
     """
     Establish WebSocket connection to a server. The connection parameters like target
@@ -497,10 +506,13 @@ def connectWS(factory, contextFactory=None, timeout=30, bindAddress=None):
 
     :param factory: The WebSocket protocol factory to be used for creating client protocol instances.
     :type factory: An :class:`autobahn.websocket.WebSocketClientFactory` instance.
+
     :param contextFactory: SSL context factory, required for secure WebSocket connections ("wss").
     :type contextFactory: A `twisted.internet.ssl.ClientContextFactory <http://twistedmatrix.com/documents/current/api/twisted.internet.ssl.ClientContextFactory.html>`_ instance.
+
     :param timeout: Number of seconds to wait before assuming the connection has failed.
     :type timeout: int
+
     :param bindAddress: A (host, port) tuple of local address to bind to, or None.
     :type bindAddress: tuple
 
@@ -521,7 +533,7 @@ def connectWS(factory, contextFactory=None, timeout=30, bindAddress=None):
 
     if factory.proxy is not None:
         factory.contextFactory = contextFactory
-        conn = reactor.connectTCP(factory.proxy['host'], factory.proxy['port'], factory, timeout, bindAddress)
+        conn = reactor.connectTCP(factory.proxy[u'host'], factory.proxy[u'port'], factory, timeout, bindAddress)
     else:
         if factory.isSecure:
             conn = reactor.connectSSL(factory.host, factory.port, factory, contextFactory, timeout, bindAddress)
@@ -530,6 +542,7 @@ def connectWS(factory, contextFactory=None, timeout=30, bindAddress=None):
     return conn
 
 
+@public
 def listenWS(factory, contextFactory=None, backlog=50, interface=''):
     """
     Listen for incoming WebSocket connections from clients. The connection parameters like
@@ -537,10 +550,13 @@ def listenWS(factory, contextFactory=None, backlog=50, interface=''):
 
     :param factory: The WebSocket protocol factory to be used for creating server protocol instances.
     :type factory: An :class:`autobahn.websocket.WebSocketServerFactory` instance.
+
     :param contextFactory: SSL context factory, required for secure WebSocket connections ("wss").
     :type contextFactory: A twisted.internet.ssl.ContextFactory.
+
     :param backlog: Size of the listen queue.
     :type backlog: int
+
     :param interface: The interface (derived from hostname given) to bind to, defaults to '' (all).
     :type interface: str
 
@@ -562,25 +578,41 @@ def listenWS(factory, contextFactory=None, backlog=50, interface=''):
     return listener
 
 
+@public
 class WampWebSocketServerProtocol(websocket.WampWebSocketServerProtocol, WebSocketServerProtocol):
     """
-    Base class for Twisted-based WAMP-over-WebSocket server protocols.
+    Twisted-based WAMP-over-WebSocket server protocol.
+
+    Implements:
+
+    * :class:`autobahn.wamp.interfaces.ITransport`
     """
 
 
+@public
 class WampWebSocketServerFactory(websocket.WampWebSocketServerFactory, WebSocketServerFactory):
     """
-    Base class for Twisted-based WAMP-over-WebSocket server factories.
+    Twisted-based WAMP-over-WebSocket server protocol factory.
     """
 
     protocol = WampWebSocketServerProtocol
 
     def __init__(self, factory, *args, **kwargs):
+        """
+
+        :param factory: A callable that produces instances that implement
+            :class:`autobahn.wamp.interfaces.ITransportHandler`
+        :type factory: callable
+
+        :param serializers: A list of WAMP serializers to use (or ``None``
+            for all available serializers).
+        :type serializers: list of objects implementing
+            :class:`autobahn.wamp.interfaces.ISerializer`
+        """
 
         serializers = kwargs.pop('serializers', None)
-        debug_wamp = kwargs.pop('debug_wamp', False)
 
-        websocket.WampWebSocketServerFactory.__init__(self, factory, serializers, debug_wamp=debug_wamp)
+        websocket.WampWebSocketServerFactory.__init__(self, factory, serializers)
 
         kwargs['protocols'] = self._protocols
 
@@ -588,25 +620,41 @@ class WampWebSocketServerFactory(websocket.WampWebSocketServerFactory, WebSocket
         WebSocketServerFactory.__init__(self, *args, **kwargs)
 
 
+@public
 class WampWebSocketClientProtocol(websocket.WampWebSocketClientProtocol, WebSocketClientProtocol):
     """
-    Base class for Twisted-based WAMP-over-WebSocket client protocols.
+    Twisted-based WAMP-over-WebSocket client protocol.
+
+    Implements:
+
+    * :class:`autobahn.wamp.interfaces.ITransport`
     """
 
 
+@public
 class WampWebSocketClientFactory(websocket.WampWebSocketClientFactory, WebSocketClientFactory):
     """
-    Base class for Twisted-based WAMP-over-WebSocket client factories.
+    Twisted-based WAMP-over-WebSocket client protocol factory.
     """
 
     protocol = WampWebSocketClientProtocol
 
     def __init__(self, factory, *args, **kwargs):
+        """
+
+        :param factory: A callable that produces instances that implement
+            :class:`autobahn.wamp.interfaces.ITransportHandler`
+        :type factory: callable
+
+        :param serializer: The WAMP serializer to use (or ``None`` for
+           "best" serializer, chosen as the first serializer available from
+           this list: CBOR, MessagePack, UBJSON, JSON).
+        :type serializer: object implementing :class:`autobahn.wamp.interfaces.ISerializer`
+        """
 
         serializers = kwargs.pop('serializers', None)
-        debug_wamp = kwargs.pop('debug_wamp', False)
 
-        websocket.WampWebSocketClientFactory.__init__(self, factory, serializers, debug_wamp=debug_wamp)
+        websocket.WampWebSocketClientFactory.__init__(self, factory, serializers)
 
         kwargs['protocols'] = self._protocols
 
