@@ -1273,6 +1273,8 @@ class ApplicationSession(BaseSession):
                 msg.correlation_uri = options.correlation_uri
             if options.correlation_is_anchor is not None:
                 msg.correlation_is_anchor = options.correlation_is_anchor
+            if options.correlation_is_last is not None:
+                msg.correlation_is_last = options.correlation_is_last
 
         if options and options.acknowledge:
             # only acknowledged publications expect a reply ..
@@ -1328,6 +1330,8 @@ class ApplicationSession(BaseSession):
                     msg.correlation_uri = options.correlation_uri
                 if options.correlation_is_anchor is not None:
                     msg.correlation_is_anchor = options.correlation_is_anchor
+                if options.correlation_is_last is not None:
+                    msg.correlation_is_last = options.correlation_is_last
 
             self._transport.send(msg)
             return on_reply
@@ -1451,6 +1455,8 @@ class ApplicationSession(BaseSession):
                 msg.correlation_uri = options.correlation_uri
             if options.correlation_is_anchor is not None:
                 msg.correlation_is_anchor = options.correlation_is_anchor
+            if options.correlation_is_last is not None:
+                msg.correlation_is_last = options.correlation_is_last
 
         # FIXME: implement call canceling
         # def canceller(_d):
@@ -1511,6 +1517,8 @@ class ApplicationSession(BaseSession):
                     msg.correlation_uri = options.correlation_uri
                 if options.correlation_is_anchor is not None:
                     msg.correlation_is_anchor = options.correlation_is_anchor
+                if options.correlation_is_last is not None:
+                    msg.correlation_is_last = options.correlation_is_last
 
             self._transport.send(msg)
             return on_reply
